//
// LCDHype output device / controller interface, requires LCDHype version 0.7.2 or above
//
// $Revision: 7 $
//
// $Id: ControllerInterface.h 7 2012-09-30 15:13:01Z LostSoul $
//

#ifndef CONTROLLERINTERFACE_H_
#define CONTROLLERINTERFACE_H_

#warning "Enum size is compiler specific, ensure size is 4 bytes"
#pragma pack(push, 4)

typedef unsigned long LongWord;
typedef char * PByte;
typedef char Byte;
typedef void * Pointer;
typedef int Integer;
typedef bool Boolean;
typedef wchar_t WideChar;
typedef long THandle;
typedef bool * PBoolean;
typedef LongWord * PLongWord;
typedef WideChar * PWideChar;

struct TControllerInterfaceCustomChar
{
  LongWord StructureSize;
  LongWord Position;       // states the position where the custom char should be stored in LCD memory
  LongWord Width;          // states the width of the custom char
  LongWord Height;         // states the height of the custom char
  PByte Pixel;             // holds the width * height custom char pixel bytes, each byte is a pixel, 0 = off, 1 = on
};
typedef TControllerInterfaceCustomChar * PControllerInterfaceCustomChar;

struct TControllerInterfaceSymbol
{
  LongWord StructureSize;
  LongWord Index;                      // symbol index
  Byte State;                          // state of a symbol, currently only 0 for off, 1 for on is supported
};
typedef TControllerInterfaceSymbol * PControllerInterfaceSymbol;

// Kind field of TControllerInterfaceInOutProperty    // Value field of TControllerInterface%Property, Description
enum TControllerInterfaceGetPropertyKind
{
  gpName = 0,                                         // in: nil, return: PWideChar, states the driver name, don't include version when library supplies version info because LCDHype reads out version info automatically
  gpTextModeResolution = 1,                           // PControllerInterfaceResolution
  gpTextModeCharacterResolution = 2,                  // PControllerInterfaceResolution
  gpFontPitch = 3,                                    // PByte, font pitch (in most cases GraphicModeWidth / TextModeWidth) of output device in pixels
  gpGraphicModeResolution = 4,                        // PControllerInterfaceResolution
  gpColorModeSupported = 5,                           // PBoolean
  gpConfigurationDialogSupported = 6,                 // PBoolean
  gpOutportCount = 7,                                 // PLongWord, number of outgoing ports supported
  gpSymbolCount = 8,                                  // PLongWord, number of supported symbols
  gpBacklightSliderIntervalCount = 9,                 // PLongWord, a numeric value >= 2 stating number of intervals used by back light slider, if back light slider is not supported then pass 0
  gpContrastSliderIntervalCount = 10,                 // PLongWord, a numeric value >= 2 stating number of intervals used by contrast slider, if contrast slider is not supported then pass 0
  gpReadyToReceive = 11,                              // PBoolean, property continuously polled, signals whether the output device is ready to receive new data or not
  gpCustomCharPosition = 12,                          // PControllerInterfaceCustomCharPosition, property requested by LCDHype to get the position of a custom char in the memory of the output device
  gpCustomCharCount = 13                              // PLongWord, number of supported custom chars
};

enum TControllerInterfaceSetPropertyKind
{
  spLibraryPath = 0,                                  // PWideChar, DLL path
  spConfigurationPath = 1,                            // PWideChar, path where configuration files should be stored
  spBacklightState = 2,                               // PBoolean, back light on = true, back light off = false
  spBacklightSliderValue = 3,                         // PLongWord
  spContrastState = 4,                                // PBoolean, contrast on = true, contrast off = false
  spContrastSliderValue = 5,                          // PLongWord
  spOutportState = 6,                                 // PControllerInterfaceOutportState
  spInvertedOutput = 7,                               // PBoolean
  spAddLogEntryCallback = 8,                          // TLogInterfaceAddEntryCallBack (see LogInterface.h), call this method to send logs to LCDHype
  spMainWindowHandle = 9,                             // PLongWord, window handle of application main window
  spOutputEnabled = 10                                // PBoolean
};

struct TControllerInterfaceGetProperty
{
  LongWord StructureSize;
  TControllerInterfaceGetPropertyKind Kind;
  Pointer Value;
};
typedef TControllerInterfaceGetProperty * PControllerInterfaceGetProperty;

struct TControllerInterfaceSetProperty
{
  LongWord StructureSize;
  TControllerInterfaceSetPropertyKind Kind;
  Pointer Value;
};
typedef TControllerInterfaceSetProperty * PControllerInterfaceSetProperty;

struct TControllerInterfaceSetPropertyList
{
  LongWord StructureSize;
  Integer Length;
  PControllerInterfaceSetProperty Element;
};
typedef TControllerInterfaceSetPropertyList * PControllerInterfaceSetPropertyList;

struct TControllerInterfaceOutportState
{
  LongWord Index;
  Boolean State;
};
typedef TControllerInterfaceOutportState * PControllerInterfaceOutportState;

struct TControllerInterfaceResolution
{
  LongWord Width;
  LongWord Height;
};
typedef TControllerInterfaceResolution * PControllerInterfaceResolution;

struct TControllerInterfaceCustomCharPosition
{
  LongWord Internal; // position internally used by LCDHype, first custom char begins at internal position 0
  LongWord Memory;   // position where custom char is stored in memory of output device,
                     // when LCDHype passes 0 as internal position and the first custom char is stored at memory position 128 then the
                     // the field Memory should be filled with 128
};
typedef TControllerInterfaceCustomCharPosition * PControllerInterfaceCustomCharPosition;

struct TControllerInterfaceTextCharacter
{
  LongWord StructureSize;
  WideChar Character;
  Boolean Underlined;
};
typedef TControllerInterfaceTextCharacter * PControllerInterfaceTextCharacter;

struct TControllerInterfaceText
{
  LongWord StructureSize;
  PControllerInterfaceTextCharacter Scan; // width * height text characters
  LongWord Width;
  LongWord Height;
  LongWord Column; // states the column where to start output, currently LCDHype always passes 0
  LongWord Row;    // states the row where to start output, currently LCDHype always passes 0
};
typedef TControllerInterfaceText * PControllerInterfaceText;

#pragma pack(push, 1)
struct TControllerInterfaceBitmapPixel
{
  Byte Blue;
  Byte Green;
  Byte Red;
  Byte Alpha; // alpha currently holds no valid value, so skip it
};
#pragma pack(pop)
typedef TControllerInterfaceBitmapPixel * PControllerInterfaceBitmapPixel;

struct TControllerInterfaceBitmap
{
  LongWord StructureSize;
  PControllerInterfaceBitmapPixel Scan;  // number of PBitmapPixel = width * height
  LongWord Width;                        // number of pixels in horizontal direction
  LongWord Height;                       // number of pixels in vertical direction
  LongWord X1;                           // defines the left boundary of the rectangle within the bitmap that should be displayed
  LongWord Y1;                           // defines the top boundary of the rectangle within the bitmap that should be displayed
  LongWord X2;                           // defines the right boundary of the rectangle within the bitmap that should be displayed
  LongWord Y2;                           // defines the bottom boundary of the rectangle within the bitmap that should be displayed
};
typedef TControllerInterfaceBitmap * PControllerInterfaceBitmap;

// Kind field of TControllerInterfaceCommand    // Value field of TControllerInterfaceCommand, Description
enum TControllerInterfaceCommandKind
{
  ckReserved = 0,                               // PByte, this kind of command is needed by very old drivers
  ckHandleBacklightOn = 1,                      // not used
  ckHandleBacklightOff = 2,                     // not used
  ckHandleContrastOn = 3,                       // not used
  ckHandleContrastOff = 4,                      // not used
  ckHandleOutportChanged = 5,                   // not used
  ckShowConfigurationDialog = 6,                // PControllerInterfaceConfigurationDialogParameter, when the user clicks
                                                // on the "Advanced configuration" button in the controller configuration dialog
  ckHandleSuspending = 7,                       // not used, machine is going into suspended state
  ckHandleResuming = 8                          // not used, machine is resuming from suspended state
};

struct TControllerInterfaceCommand
{
  LongWord StructureSize;
  TControllerInterfaceCommandKind Kind;
  Pointer Value;
};
typedef TControllerInterfaceCommand * PControllerInterfaceCommand;

struct TControllerInterfaceConfigurationDialogParameter
{
  LongWord StructureSize;
  THandle ParentWindow;
};
typedef TControllerInterfaceConfigurationDialogParameter * PControllerInterfaceConfigurationDialogParameter;

struct TControllerInterfaceInitParameter
{
  LongWord StructureSize;
  LongWord CallCount;
};
typedef TControllerInterfaceInitParameter * PControllerInterfaceInitParameter;

struct TControllerInterfaceCleanUpParameter
{
  LongWord StructureSize;
};
typedef TControllerInterfaceCleanUpParameter * PControllerInterfaceCleanUpParameter;

#pragma pack(pop)

// method is called by LCDHype to get information from the driver, e.g. name or configuration,
// please have a look at TControllerInterfaceGetPropertyKind to see which properties are supported
typedef void (__stdcall * TControllerInterfaceGetPropertyMethod)(PControllerInterfaceGetProperty AProperty);

// method is called by LCDHype to set several properties that may be needed by the driver to work correctly,
// please have a look at TControllerInterfaceSetPropertyKind to see which properties are supported
typedef void (__stdcall * TControllerInterfaceSetPropertyMethod)(PControllerInterfaceSetPropertyList AProperty);

// method is called by LCDHype to initialize the controller
typedef void (__stdcall * TControllerInterfaceInitMethod)(PControllerInterfaceInitParameter AProperty);

// method is called by LCDHype to free up resources used by library
typedef void (__stdcall * TControllerInterfaceCleanUpMethod)(PControllerInterfaceCleanUpParameter AProperty);

// method is called by LCDHype to send a custom char to the output device
typedef void (__stdcall * TControllerInterfaceSendCustomCharMethod)(PControllerInterfaceCustomChar AProperty);

// method is called by LCDHype to send a command to the output device, return true when command is being processed
typedef Boolean (__stdcall * TControllerInterfaceSendCommandMethod)(PControllerInterfaceCommand AProperty);

// method is called by LCDHype to send text the output device
typedef void (__stdcall * TControllerInterfaceSendTextMethod)(PControllerInterfaceText AProperty);

// method is called by LCDHype to send a bitmap the output device
typedef void (__stdcall * TControllerInterfaceSendBitmapMethod)(PControllerInterfaceBitmap AProperty);

// method is called by LCDHype to refresh a symbol displayed by the output device
typedef void (__stdcall * TControllerInterfaceSendSymbolMethod)(PControllerInterfaceSymbol AProperty);

#endif
